import webpackChain from 'webpack-chain';
import { IPluginAPI as IPluginAPI$1, IFastHookRegister, IArgs, IRawArgs, IPathManager, IEnvManager, IConfigManager, IPluginManager, IProjectConfig, IPlugin as IPlugin$1 } from '@xus/core';
export { IArgs, ICommand, IRawArgs } from '@xus/core';
import { IEsbuildRegister, IRunCmdMessage } from '@xus/cli-shared';
import { IRollupChain, IRollupChainConfig } from '@xus/rollup-chain';
export { IRollupChain, IRollupChainConfig } from '@xus/rollup-chain';
import { TransformOptions } from 'esbuild';
import { ProcessOptions, Plugin } from 'postcss';

declare type noopFn = () => any;
interface IPluginAPI extends IPluginAPI$1 {
    EsbuildRegister: IEsbuildRegister;
    onSetuped: IFastHookRegister<(config: IConfig) => void>;
    onRunCmd: noopFn;
    getCmdArgs: () => {
        args: IArgs;
        rawArgs: IRawArgs;
    };
    modifyProjectConfig: (config: IConfig) => void;
    cwd: IPathManager['cwd'];
    cwdPkg: IPathManager['cwdPkg'];
    userConfigPath: IPathManager['userConfigPath'];
    getPathBasedOnCtx: IPathManager['getPathBasedOnCtx'];
    getLernaPkgs: IPathManager['getLernaPkgs'];
    mode: IEnvManager['mode'];
    babelEnv: IEnvManager['babelEnv'];
    getEnv: IEnvManager['getEnv'];
    setEnv: IEnvManager['setEnv'];
    getCliEnv: IEnvManager['getCliEnv'];
    setCliEnv: IEnvManager['setCliEnv'];
    projectConfig: IConfig;
    cwdPkgJson: IConfigManager['cwdPkgJson'];
    loadConfig: IConfigManager['loadConfig'];
    skipPlugin: IPluginManager['skipPlugin'];
    modifyRollupConfig: IFastHookRegister<(rc: IRollupChain, arg?: any) => IRollupChain>;
    getRollupConfig: (arg?: any) => Promise<IRollupChainConfig>;
    modifyWebpackConfig: IFastHookRegister<(wc: webpackChain, arg?: any) => webpackChain>;
    getWebpackConfig: (arg?: any) => Promise<any>;
}
declare type ICmd = {
    bin: string;
    args: string[];
    message: IRunCmdMessage;
};
interface IConfig extends IProjectConfig {
    libBuild: {
        libName: string;
        target: 'esnext' | TransformOptions['target'];
        formats: ('esm' | 'cjs' | 'iife' | 'umd')[];
        disableFormatPostfix: boolean;
        rollTypes: boolean;
        sourcemap: boolean;
        minify: false | 'esbuild' | 'terser';
        alwaysEmptyDistDir: boolean;
        css: {
            injectMode: false | 'script' | 'url';
            cssCodeSplit: boolean;
            modules: {
                getJSON?: (cssFileName: string, json: Record<string, string>, outputFileName: string) => void;
                scopeBehaviour?: 'global' | 'local';
                globalModulePaths?: string[];
                generateScopedName?: string | ((name: string, filename: string, css: string) => string);
                hashPrefix?: string;
                /**
                 * default: null
                 */
                localsConvention?: 'camelCase' | 'camelCaseOnly' | 'dashes' | 'dashesOnly' | null;
            };
            postcss: ProcessOptions & {
                plugins: Plugin[];
            };
            preprocessor: Partial<Record<'sass' | 'less' | 'stylus', any>>;
        };
        assets: {
            dirname: string;
            inlineLimit: number;
        };
        json: {
            exportMode: 'named' | 'stringify';
        };
        alias: Record<string, string>;
        replace: Record<string, string>;
        excludeExternal: string[];
        external: string[];
        lerna: false | {
            pkgsOrder: string[];
            excludePkgs: string[];
        };
        rollupChain: (rc: IRollupChain, pkgDir: string) => IRollupChain;
        afterBuild: ICmd[];
    };
    lint: {
        eslint: boolean | {
            include?: string[];
            ext?: string[];
        };
        stylelint: boolean | {
            include?: string[];
        };
    };
    changelog: {
        filename: string;
        mainTemplate: string;
        headerPartial: string;
        commitPartial: string;
    };
    release: {
        beforeRelease: ICmd[];
        changelog: boolean;
        registry: string;
        order: string[];
        branch: string;
    };
}
declare type IPlugin = IPlugin$1<(api: IPluginAPI) => void>;
interface IPreset {
    plugins: IPlugin[];
}

declare type IReturnArg<T extends any> = (args: T) => T;
declare type PartialDeep<T extends Record<string, any>> = {
    [K in keyof T]?: T[K] extends Record<string, unknown> | boolean ? PartialDeep<T[K]> : T[K];
};
declare const defineConfig: IReturnArg<PartialDeep<IConfig>>;
declare const createPlugin: IReturnArg<IPlugin>;
declare const createPreset: IReturnArg<IPreset>;

declare type IWebpackChain = webpackChain;

export { IConfig, IPlugin, IPluginAPI, IPreset, IWebpackChain, createPlugin, createPreset, defineConfig };
